@@ -5,11 +5,22 @@ from __future__ import print_function
 import sys
 import tempfile
 import os
+import unittest
 
-if sys.version_info[:2] <= (2, 6):
-    import unittest2 as unittest
-else:
-    import unittest
+from PIL import Image, ImageMath
+
+
+def convert_to_comparable(a, b):
+    new_a, new_b = a, b
+    if a.mode == 'P':
+        new_a = Image.new('L', a.size)
+        new_b = Image.new('L', b.size)
+        new_a.putdata(a.getdata())
+        new_b.putdata(b.getdata())
+    elif a.mode == 'I;16':
+        new_a = a.convert('I')
+        new_b = b.convert('I')
+    return new_a, new_b
 
 
 class PillowTestCase(unittest.TestCase):
@@ -49,7 +60,7 @@ class PillowTestCase(unittest.TestCase):
                 len(a), len(b),
                 msg or "got length %s, expected %s" % (len(a), len(b)))
             self.assertTrue(
-                all([x == y for x, y in zip(a, b)]),
+                all(x == y for x, y in zip(a, b)),
                 msg or "got %s, expected %s" % (a, b))
         except:
             self.assertEqual(a, b, msg)
@@ -84,14 +95,13 @@ class PillowTestCase(unittest.TestCase):
             a.size, b.size,
             msg or "got size %r, expected %r" % (a.size, b.size))
 
+        a, b = convert_to_comparable(a, b)
+
         diff = 0
-        try:
-            ord(b'0')
-            for abyte, bbyte in zip(a.tobytes(), b.tobytes()):
-                diff += abs(ord(abyte)-ord(bbyte))
-        except:
-            for abyte, bbyte in zip(a.tobytes(), b.tobytes()):
-                diff += abs(abyte-bbyte)
+        for ach, bch in zip(a.split(), b.split()):
+            chdiff = ImageMath.eval("abs(a - b)", a=ach, b=bch).convert('L')
+            diff += sum(i * num for i, num in enumerate(chdiff.histogram()))
+
         ave_diff = float(diff)/(a.size[0]*a.size[1])
         self.assertGreaterEqual(
             epsilon, ave_diff,
