@@ -1,3 +1,4 @@
+from __future__ import print_function
 from helper import unittest, PillowTestCase
 
 from PIL import Image
@@ -21,18 +22,32 @@ class TestBmpReference(PillowTestCase):
                 im.load()
             except Exception:  # as msg:
                 pass
-                # print ("Bad Image %s: %s" %(f,msg))
+                # print("Bad Image %s: %s" %(f,msg))
 
     def test_questionable(self):
         """ These shouldn't crash/dos, but it's not well defined that these
         are in spec """
+        supported = [
+            "pal8os2v2.bmp",
+            "rgb24prof.bmp",
+            "pal1p1.bmp",
+            "pal8offs.bmp",
+            "rgb24lprof.bmp",
+            "rgb32fakealpha.bmp",
+            "rgb24largepal.bmp",
+            "pal8os2sp.bmp",
+            "rgb32bf-xbgr.bmp",
+        ]
         for f in self.get_files('q'):
             try:
                 im = Image.open(f)
                 im.load()
+                if os.path.basename(f) not in supported:
+                    print("Please add %s to the partially supported bmp specs." % f)
             except Exception:  # as msg:
-                pass
-                # print ("Bad Image %s: %s" %(f,msg))
+                if os.path.basename(f) in supported:
+                    raise
+                # print("Bad Image %s: %s" %(f,msg))
 
     def test_good(self):
         """ These should all work. There's a set of target files in the
@@ -90,5 +105,3 @@ class TestBmpReference(PillowTestCase):
 
 if __name__ == '__main__':
     unittest.main()
-
-# End of file
