@@ -6,7 +6,6 @@ from PIL import Image, FliImagePlugin
 # created as an export of a palette image from Gimp2.6
 # save as...-> hopper.fli, default options.
 test_file = "Tests/images/hopper.fli"
-data = open(test_file, "rb").read()
 
 
 class TestFileFli(PillowTestCase):
@@ -44,5 +43,3 @@ class TestFileFli(PillowTestCase):
 
 if __name__ == '__main__':
     unittest.main()
-
-# End of file
