@@ -12,7 +12,6 @@ codecs = dir(Image.core)
 # sample png stream
 
 TEST_PNG_FILE = "Tests/images/hopper.png"
-TEST_DATA = open(TEST_PNG_FILE, "rb").read()
 
 # stuff to create inline PNG images
 
@@ -55,7 +54,7 @@ class TestFilePng(PillowTestCase):
 
         # internal version number
         self.assertRegexpMatches(
-            Image.core.zlib_version, "\d+\.\d+\.\d+(\.\d+)?$")
+            Image.core.zlib_version, r"\d+\.\d+\.\d+(\.\d+)?$")
 
         test_file = self.tempfile("temp.png")
 
@@ -198,7 +197,7 @@ class TestFilePng(PillowTestCase):
         im = im.convert("RGBA")
         self.assert_image(im, "RGBA", (162, 150))
 
-        # image has 124 uniqe qlpha values
+        # image has 124 unique alpha values
         self.assertEqual(len(im.split()[3].getcolors()), 124)
 
     def test_load_transparent_rgb(self):
@@ -313,7 +312,7 @@ class TestFilePng(PillowTestCase):
         #                  -14: malformed chunk
 
         for offset in (-10, -13, -14):
-            with open(TEST_PNG_FILE,'rb') as f:
+            with open(TEST_PNG_FILE, 'rb') as f:
                 test_file = f.read()[:offset]
 
             im = Image.open(BytesIO(test_file))
@@ -347,7 +346,6 @@ class TestFilePng(PillowTestCase):
         finally:
             ImageFile.LOAD_TRUNCATED_IMAGES = False
 
-
     def test_roundtrip_dpi(self):
         # Check dpi roundtripping
 
@@ -393,7 +391,7 @@ class TestFilePng(PillowTestCase):
         info = PngImagePlugin.PngInfo()
         info.add_text("Text", "Ascii")
         im = roundtrip(im, pnginfo=info)
-        self.assertEqual(type(im.info["Text"]), str)
+        self.assertIsInstance(im.info["Text"], str)
 
     def test_unicode_text(self):
         # Check preservation of non-ASCII characters on Python3
@@ -502,5 +500,3 @@ class TestFilePng(PillowTestCase):
 
 if __name__ == '__main__':
     unittest.main()
-
-# End of file
