@@ -63,6 +63,18 @@ class TestImage(PillowTestCase):
             os.remove(temp_file)
         im.save(Path(temp_file))
 
+    def test_fp_name(self):
+        temp_file = self.tempfile("temp.jpg")
+
+        class FP(object):
+            def write(a, b):
+                pass
+        fp = FP()
+        fp.name = temp_file
+
+        im = hopper()
+        im.save(fp)
+
     def test_tempfile(self):
         # see #1460, pathlib support breaks tempfile.TemporaryFile on py27
         # Will error out on save on 3.0.0
@@ -237,7 +249,33 @@ class TestImage(PillowTestCase):
         im3 = Image.open('Tests/images/effect_spread.png')
         self.assert_image_similar(im2, im3, 110)
 
+    def test_check_size(self):
+        # Checking that the _check_size function throws value errors when we want it to.
+        with self.assertRaises(ValueError):
+            Image.new('RGB', 0)  # not a tuple
+        with self.assertRaises(ValueError):
+            Image.new('RGB', (0,))  # Tuple too short
+        with self.assertRaises(ValueError):
+            Image.new('RGB', (-1,-1))  # w,h < 0
+
+        # this should pass with 0 sized images, #2259
+        im = Image.new('L', (0, 0))
+        self.assertEqual(im.size, (0, 0))
+
+        self.assertTrue(Image.new('RGB', (1,1)))
+        # Should pass lists too
+        i = Image.new('RGB', [1,1])
+        self.assertIsInstance(i.size, tuple)
+
+    def test_storage_neg(self):
+        # Storage.c accepted negative values for xsize, ysize.  Was
+        # test_neg_ppm, but the core function for that has been
+        # removed Calling directly into core to test the error in
+        # Storage.c, rather than the size check above
+
+        with self.assertRaises(ValueError):
+            Image.core.fill('RGB', (2,-2), (0,0,0))
+
+
 if __name__ == '__main__':
     unittest.main()
-
-# End of file
