@@ -1,4 +1,4 @@
-from helper import PillowTestCase, cached_property
+from helper import unittest, PillowTestCase, cached_property
 
 from PIL import Image
 
@@ -250,3 +250,14 @@ class TestImagingPaste(PillowTestCase):
                 (48, 15, 15, 47),
                 (126, 63, 255, 63)
             ])
+
+    def test_different_sizes(self):
+        im = Image.new('RGB', (100, 100))
+        im2 = Image.new('RGB', (50, 50))
+
+        im.copy().paste(im2)
+        im.copy().paste(im2, (0,0))
+
+
+if __name__ == '__main__':
+    unittest.main()
