@@ -71,7 +71,7 @@
  * See the README file for information on usage and redistribution.
  */
 
-#define PILLOW_VERSION "3.1.2"
+#define PILLOW_VERSION "4.0.0"
 
 #include "Python.h"
 
@@ -251,7 +251,9 @@ int PyImaging_GetBuffer(PyObject* buffer, Py_buffer *view)
     /* Use new buffer protocol if available
        (mmap doesn't support this in 2.7, go figure) */
     if (PyObject_CheckBuffer(buffer)) {
-        return PyObject_GetBuffer(buffer, view, PyBUF_SIMPLE);
+        int success = PyObject_GetBuffer(buffer, view, PyBUF_SIMPLE);
+        if (!success) { return success; }
+        PyErr_Clear();
     }
 
     /* Pretend we support the new protocol; PyBuffer_Release happily ignores
@@ -280,6 +282,8 @@ int PyImaging_GetBuffer(PyObject* buffer, Py_buffer *view)
 
 /* error messages */
 static const char* must_be_sequence = "argument must be a sequence";
+static const char* must_be_two_coordinates =
+                     "coordinate list must contain exactly 2 coordinates";
 static const char* wrong_mode = "unrecognized image mode";
 static const char* wrong_raw_mode = "unrecognized raw mode";
 static const char* outside_image = "image index out of range";
@@ -360,9 +364,20 @@ getbands(const char* mode)
 #define TYPE_DOUBLE (0x400|sizeof(double))
 
 static void*
-getlist(PyObject* arg, int* length, const char* wrong_length, int type)
+getlist(PyObject* arg, Py_ssize_t* length, const char* wrong_length, int type)
 {
-    int i, n, itemp;
+    /* - allocates and returns a c array of the items in the
+          python sequence arg.
+       - the size of the returned array is in length
+       - all of the arg items must be numeric items of the type
+          specified in type
+       - sequence length is checked against the length parameter IF
+          an error parameter is passed in wrong_length
+       - caller is responsible for freeing the memory
+    */
+
+    Py_ssize_t i, n;
+    int itemp;
     double dtemp;
     void* list;
     PyObject* seq;
@@ -379,7 +394,9 @@ getlist(PyObject* arg, int* length, const char* wrong_length, int type)
         return NULL;
     }
 
-    list = malloc(n * (type & 0xff));
+    /* malloc check ok, type & ff is just a sizeof(something)
+       calloc checks for overflow */
+    list = calloc(n, type & 0xff);
     if (!list)
         return PyErr_NoMemory();
 
@@ -471,29 +488,36 @@ getpixel(Imaging im, ImagingAccess access, int x, int y)
 static char*
 getink(PyObject* color, Imaging im, char* ink)
 {
-    int r, g, b, a;
-    double f;
+    int g=0, b=0, a=0;
+    double f=0;
+    /* Windows 64 bit longs are 32 bits, and 0xFFFFFFFF (white) is a
+       python long (not int) that raises an overflow error when trying
+       to return it into a 32 bit C long
+    */
+    PY_LONG_LONG r = 0;
 
     /* fill ink buffer (four bytes) with something that can
        be cast to either UINT8 or INT32 */
 
-    int rIsInt = 1;
+    int rIsInt = 0;
     if (im->type == IMAGING_TYPE_UINT8 ||
         im->type == IMAGING_TYPE_INT32 ||
         im->type == IMAGING_TYPE_SPECIAL) {
 #if PY_VERSION_HEX >= 0x03000000
-		if (PyLong_Check(color)) {
-			r = (int) PyLong_AsLong(color);
+                if (PyLong_Check(color)) {
+                        r = PyLong_AsLongLong(color);
 #else
-		if (PyInt_Check(color) || PyLong_Check(color)) {
-			if (PyInt_Check(color))
-				r = PyInt_AS_LONG(color);
-			else
-				r = (int) PyLong_AsLong(color);
+                if (PyInt_Check(color) || PyLong_Check(color)) {
+                        if (PyInt_Check(color))
+                                r = PyInt_AS_LONG(color);
+                        else
+                                r = PyLong_AsLongLong(color);
 #endif
-		}
-		if (r == -1 && PyErr_Occurred())
-		    rIsInt = 0;
+            rIsInt = 1;
+                }
+                if (r == -1 && PyErr_Occurred()) {
+                    rIsInt = 0;
+        }
     }
 
     switch (im->type) {
@@ -501,23 +525,16 @@ getink(PyObject* color, Imaging im, char* ink)
         /* unsigned integer */
         if (im->bands == 1) {
             /* unsigned integer, single layer */
-            if (rIsInt != 1)
-                return NULL;
+            if (rIsInt != 1) {
+                if (!PyArg_ParseTuple(color, "L", &r)) {
+                    return NULL;
+                }
+            }
             ink[0] = CLIP(r);
             ink[1] = ink[2] = ink[3] = 0;
         } else {
             a = 255;
-#if PY_VERSION_HEX >= 0x03000000
-            if (PyLong_Check(color)) {
-                r = (int) PyLong_AsLong(color);
-#else
-            if (PyInt_Check(color) || PyLong_Check(color)) {
-                if (PyInt_Check(color))
-                    r = PyInt_AS_LONG(color);
-                else
-                    r = (int) PyLong_AsLong(color);
-#endif
-
+            if (rIsInt) {
                 /* compatibility: ABGR */
                 a = (UINT8) (r >> 24);
                 b = (UINT8) (r >> 16);
@@ -525,11 +542,11 @@ getink(PyObject* color, Imaging im, char* ink)
                 r = (UINT8) r;
             } else {
                 if (im->bands == 2) {
-                    if (!PyArg_ParseTuple(color, "i|i", &r, &a))
+                    if (!PyArg_ParseTuple(color, "L|i", &r, &a))
                         return NULL;
                     g = b = r;
                 } else {
-                    if (!PyArg_ParseTuple(color, "iii|i", &r, &g, &b, &a))
+                    if (!PyArg_ParseTuple(color, "Lii|i", &r, &g, &b, &a))
                         return NULL;
                 }
             }
@@ -590,13 +607,14 @@ _fill(PyObject* self, PyObject* args)
     if (!im)
         return NULL;
 
+    buffer[0] = buffer[1] = buffer[2] = buffer[3] = 0;
     if (color) {
         if (!getink(color, im, buffer)) {
             ImagingDelete(im);
             return NULL;
         }
-    } else
-        buffer[0] = buffer[1] = buffer[2] = buffer[3] = 0;
+    }
+
 
     (void) ImagingFill(im, buffer);
 
@@ -670,17 +688,6 @@ _radial_gradient(PyObject* self, PyObject* args)
     return PyImagingNew(ImagingFillRadialGradient(mode));
 }
 
-static PyObject*
-_open_ppm(PyObject* self, PyObject* args)
-{
-    char* filename;
-
-    if (!PyArg_ParseTuple(args, "s", &filename))
-        return NULL;
-
-    return PyImagingNew(ImagingOpenPPM(filename));
-}
-
 static PyObject*
 _alpha_composite(ImagingObject* self, PyObject* args)
 {
@@ -842,7 +849,7 @@ static PyObject*
 _filter(ImagingObject* self, PyObject* args)
 {
     PyObject* imOut;
-    int kernelsize;
+    Py_ssize_t kernelsize;
     FLOAT32* kerneldata;
 
     int xsize, ysize;
@@ -856,7 +863,7 @@ _filter(ImagingObject* self, PyObject* args)
     kerneldata = getlist(kernel, &kernelsize, NULL, TYPE_FLOAT32);
     if (!kerneldata)
         return NULL;
-    if (kernelsize != xsize * ysize) {
+    if (kernelsize != (Py_ssize_t) xsize * (Py_ssize_t) ysize) {
         free(kerneldata);
         return ImagingError_ValueError("bad kernel size");
     }
@@ -1145,8 +1152,8 @@ _point(ImagingObject* self, PyObject* args)
 {
     static const char* wrong_number = "wrong number of lut entries";
 
-    int n, i;
-    int bands;
+    Py_ssize_t n;
+    int i, bands;
     Imaging im;
 
     PyObject* list;
@@ -1342,6 +1349,8 @@ _putdata(ImagingObject* self, PyObject* args)
                     INT32 inkint;
                 } u;
 
+                u.inkint = 0;
+
                 op = PySequence_Fast_GET_ITEM(seq, i);
                 if (!op || !getink(op, image, u.ink)) {
                     Py_DECREF(seq);
@@ -1539,17 +1548,17 @@ _resize(ImagingObject* self, PyObject* args)
     if (imIn->xsize == xsize && imIn->ysize == ysize) {
         imOut = ImagingCopy(imIn);
     }
-    else if ( ! filter) {
+    else if (filter == IMAGING_TRANSFORM_NEAREST) {
         double a[6];
 
         memset(a, 0, sizeof a);
-        a[1] = (double) imIn->xsize / xsize;
-        a[5] = (double) imIn->ysize / ysize;
+        a[0] = (double) imIn->xsize / xsize;
+        a[4] = (double) imIn->ysize / ysize;
 
         imOut = ImagingNew(imIn->mode, xsize, ysize);
 
-        imOut = ImagingTransformAffine(
-            imOut, imIn,
+        imOut = ImagingTransform(
+            imOut, imIn, IMAGING_TRANSFORM_AFFINE,
             0, 0, xsize, ysize,
             a, filter, 1);
     }
@@ -1560,55 +1569,6 @@ _resize(ImagingObject* self, PyObject* args)
     return PyImagingNew(imOut);
 }
 
-static PyObject*
-_rotate(ImagingObject* self, PyObject* args)
-{
-    Imaging imOut;
-    Imaging imIn;
-
-    double theta;
-    int filter = IMAGING_TRANSFORM_NEAREST;
-    int expand;
-    if (!PyArg_ParseTuple(args, "d|i|i", &theta, &filter, &expand))
-        return NULL;
-
-    imIn = self->image;
-
-    theta = fmod(theta, 360.0);
-    if (theta < 0.0)
-    theta += 360;
-
-    if (filter && imIn->type != IMAGING_TYPE_SPECIAL) {
-        /* Rotate with resampling filter */
-        imOut = ImagingNew(imIn->mode, imIn->xsize, imIn->ysize);
-    (void) ImagingRotate(imOut, imIn, theta, filter);
-    } else if ((theta == 90.0 || theta == 270.0)
-            && (expand || imIn->xsize == imIn->ysize)) {
-        /* Use fast version */
-        imOut = ImagingNew(imIn->mode, imIn->ysize, imIn->xsize);
-        if (imOut) {
-            if (theta == 90.0)
-                (void) ImagingRotate90(imOut, imIn);
-            else
-                (void) ImagingRotate270(imOut, imIn);
-        }
-    } else {
-        imOut = ImagingNew(imIn->mode, imIn->xsize, imIn->ysize);
-        if (imOut) {
-            if (theta == 0.0)
-                /* No rotation: simply copy the input image */
-                (void) ImagingCopy2(imOut, imIn);
-            else if (theta == 180.0)
-                /* Use fast version */
-                (void) ImagingRotate180(imOut, imIn);
-            else
-                /* Use ordinary version */
-                (void) ImagingRotate(imOut, imIn, theta, 0);
-        }
-    }
-
-    return PyImagingNew(imOut);
-}
 
 #define IS_RGB(mode)\
     (!strcmp(mode, "RGB") || !strcmp(mode, "RGBA") || !strcmp(mode, "RGBX"))
@@ -1657,9 +1617,8 @@ _transform2(ImagingObject* self, PyObject* args)
 {
     static const char* wrong_number = "wrong number of matrix entries";
 
-    Imaging imIn;
     Imaging imOut;
-    int n;
+    Py_ssize_t n;
     double *a;
 
     ImagingObject* imagep;
@@ -1693,30 +1652,9 @@ _transform2(ImagingObject* self, PyObject* args)
     if (!a)
         return NULL;
 
-    imOut = self->image;
-    imIn = imagep->image;
-
-    /* FIXME: move transform dispatcher into libImaging */
-
-    switch (method) {
-    case IMAGING_TRANSFORM_AFFINE:
-        imOut = ImagingTransformAffine(
-            imOut, imIn, x0, y0, x1, y1, a, filter, 1
-            );
-        break;
-    case IMAGING_TRANSFORM_PERSPECTIVE:
-        imOut = ImagingTransformPerspective(
-            imOut, imIn, x0, y0, x1, y1, a, filter, 1
-            );
-        break;
-    case IMAGING_TRANSFORM_QUAD:
-        imOut = ImagingTransformQuad(
-            imOut, imIn, x0, y0, x1, y1, a, filter, 1
-            );
-        break;
-    default:
-        (void) ImagingError_ValueError("bad transform method");
-    }
+    imOut = ImagingTransform(
+        self->image, imagep->image, method,
+        x0, y0, x1, y1, a, filter, 1);
 
     free(a);
 
@@ -1915,6 +1853,7 @@ _getprojection(ImagingObject* self, PyObject* args)
     unsigned char* yprofile;
     PyObject* result;
 
+    /* malloc check ok */
     xprofile = malloc(self->image->xsize);
     yprofile = malloc(self->image->ysize);
 
@@ -2361,7 +2300,7 @@ _draw_dealloc(ImagingDrawObject* self)
     PyObject_Del(self);
 }
 
-extern int PyPath_Flatten(PyObject* data, double **xy);
+extern Py_ssize_t PyPath_Flatten(PyObject* data, double **xy);
 
 static PyObject*
 _draw_ink(ImagingDrawObject* self, PyObject* args)
@@ -2382,7 +2321,7 @@ static PyObject*
 _draw_arc(ImagingDrawObject* self, PyObject* args)
 {
     double* xy;
-    int n;
+    Py_ssize_t n;
 
     PyObject* data;
     int ink;
@@ -2395,9 +2334,7 @@ _draw_arc(ImagingDrawObject* self, PyObject* args)
     if (n < 0)
         return NULL;
     if (n != 2) {
-        PyErr_SetString(PyExc_TypeError,
-                        "coordinate list must contain exactly 2 coordinates"
-                        );
+        PyErr_SetString(PyExc_TypeError, must_be_two_coordinates);
         return NULL;
     }
 
@@ -2420,7 +2357,7 @@ static PyObject*
 _draw_bitmap(ImagingDrawObject* self, PyObject* args)
 {
     double *xy;
-    int n;
+    Py_ssize_t n;
 
     PyObject *data;
     ImagingObject* bitmap;
@@ -2456,7 +2393,7 @@ static PyObject*
 _draw_chord(ImagingDrawObject* self, PyObject* args)
 {
     double* xy;
-    int n;
+    Py_ssize_t n;
 
     PyObject* data;
     int ink, fill;
@@ -2469,9 +2406,7 @@ _draw_chord(ImagingDrawObject* self, PyObject* args)
     if (n < 0)
         return NULL;
     if (n != 2) {
-        PyErr_SetString(PyExc_TypeError,
-                        "coordinate list must contain exactly 2 coordinates"
-                        );
+        PyErr_SetString(PyExc_TypeError, must_be_two_coordinates);
         return NULL;
     }
 
@@ -2494,7 +2429,7 @@ static PyObject*
 _draw_ellipse(ImagingDrawObject* self, PyObject* args)
 {
     double* xy;
-    int n;
+    Py_ssize_t n;
 
     PyObject* data;
     int ink;
@@ -2506,9 +2441,7 @@ _draw_ellipse(ImagingDrawObject* self, PyObject* args)
     if (n < 0)
         return NULL;
     if (n != 2) {
-        PyErr_SetString(PyExc_TypeError,
-                        "coordinate list must contain exactly 2 coordinates"
-                        );
+        PyErr_SetString(PyExc_TypeError, must_be_two_coordinates);
         return NULL;
     }
 
@@ -2547,7 +2480,7 @@ static PyObject*
 _draw_lines(ImagingDrawObject* self, PyObject* args)
 {
     double *xy;
-    int i, n;
+    Py_ssize_t i, n;
 
     PyObject *data;
     int ink;
@@ -2615,7 +2548,7 @@ static PyObject*
 _draw_points(ImagingDrawObject* self, PyObject* args)
 {
     double *xy;
-    int i, n;
+    Py_ssize_t i, n;
 
     PyObject *data;
     int ink;
@@ -2677,7 +2610,7 @@ static PyObject*
 _draw_pieslice(ImagingDrawObject* self, PyObject* args)
 {
     double* xy;
-    int n;
+    Py_ssize_t n;
 
     PyObject* data;
     int ink, fill;
@@ -2689,9 +2622,7 @@ _draw_pieslice(ImagingDrawObject* self, PyObject* args)
     if (n < 0)
         return NULL;
     if (n != 2) {
-        PyErr_SetString(PyExc_TypeError,
-                        "coordinate list must contain exactly 2 coordinates"
-                        );
+        PyErr_SetString(PyExc_TypeError, must_be_two_coordinates);
         return NULL;
     }
 
@@ -2715,7 +2646,7 @@ _draw_polygon(ImagingDrawObject* self, PyObject* args)
 {
     double *xy;
     int *ixy;
-    int n, i;
+    Py_ssize_t n, i;
 
     PyObject* data;
     int ink;
@@ -2734,7 +2665,7 @@ _draw_polygon(ImagingDrawObject* self, PyObject* args)
     }
 
     /* Copy list of vertices to array */
-    ixy = (int*) malloc(n * 2 * sizeof(int));
+    ixy = (int*) calloc(n, 2 * sizeof(int));
 
     for (i = 0; i < n; i++) {
         ixy[i+i] = (int) xy[i+i];
@@ -2759,7 +2690,7 @@ static PyObject*
 _draw_rectangle(ImagingDrawObject* self, PyObject* args)
 {
     double* xy;
-    int n;
+    Py_ssize_t n;
 
     PyObject* data;
     int ink;
@@ -2771,9 +2702,7 @@ _draw_rectangle(ImagingDrawObject* self, PyObject* args)
     if (n < 0)
         return NULL;
     if (n != 2) {
-        PyErr_SetString(PyExc_TypeError,
-                        "coordinate list must contain exactly 2 coordinates"
-                        );
+        PyErr_SetString(PyExc_TypeError, must_be_two_coordinates);
         return NULL;
     }
 
@@ -3050,10 +2979,6 @@ static struct PyMethodDef methods[] = {
     {"rankfilter", (PyCFunction)_rankfilter, 1},
 #endif
     {"resize", (PyCFunction)_resize, 1},
-    // There were two methods for image resize before.
-    // Starting from Pillow 2.7.0 stretch is depreciated.
-    {"stretch", (PyCFunction)_resize, 1},
-    {"rotate", (PyCFunction)_rotate, 1},
     {"transpose", (PyCFunction)_transpose, 1},
     {"transform2", (PyCFunction)_transform2, 1},
 
@@ -3147,11 +3072,7 @@ _getattr_id(ImagingObject* self, void* closure)
 static PyObject*
 _getattr_ptr(ImagingObject* self, void* closure)
 {
-#if (PY_VERSION_HEX >= 0x02070000 && PY_VERSION_HEX < 0x03000000) || PY_VERSION_HEX >= 0x03010000
     return PyCapsule_New(self->image, IMAGING_MAGIC, NULL);
-#else
-    return PyCObject_FromVoidPtrAndDesc(self->image, IMAGING_MAGIC, NULL);
-#endif
 }
 
 static PyObject*
@@ -3360,6 +3281,7 @@ static PyTypeObject PixelAccess_Type = {
    pluggable codecs, but not before PIL 1.2 */
 
 /* Decoders (in decode.c) */
+extern PyObject* PyImaging_BcnDecoderNew(PyObject* self, PyObject* args);
 extern PyObject* PyImaging_BitDecoderNew(PyObject* self, PyObject* args);
 extern PyObject* PyImaging_FliDecoderNew(PyObject* self, PyObject* args);
 extern PyObject* PyImaging_GifDecoderNew(PyObject* self, PyObject* args);
@@ -3425,6 +3347,7 @@ static PyMethodDef functions[] = {
     {"copy", (PyCFunction)_copy2, 1},
 
     /* Codecs */
+    {"bcn_decoder", (PyCFunction)PyImaging_BcnDecoderNew, 1},
     {"bit_decoder", (PyCFunction)PyImaging_BitDecoderNew, 1},
     {"eps_encoder", (PyCFunction)PyImaging_EpsEncoderNew, 1},
     {"fli_decoder", (PyCFunction)PyImaging_FliDecoderNew, 1},
@@ -3485,9 +3408,6 @@ static PyMethodDef functions[] = {
     {"crc32", (PyCFunction)_crc32, 1},
     {"getcodecstatus", (PyCFunction)_getcodecstatus, 1},
 
-    /* Debugging stuff */
-    {"open_ppm", (PyCFunction)_open_ppm, 1},
-
     /* Special effects (experimental) */
 #ifdef WITH_EFFECTS
     {"effect_mandelbrot", (PyCFunction)_effect_mandelbrot, 1},
