@@ -191,14 +191,64 @@ unpack1IR(UINT8* out, const UINT8* in, int pixels)
 static void
 unpackL2(UINT8* out, const UINT8* in, int pixels)
 {
-    /* nibbles */
+    /* nibbles (msb first, white is non-zero) */
     while (pixels > 0) {
         UINT8 byte = *in++;
         switch (pixels) {
-        default:    *out++ = ((byte >> 6) & 3) * 255 / 3; byte <<= 2;
-        case 3:     *out++ = ((byte >> 6) & 3) * 255 / 3; byte <<= 2;
-        case 2:     *out++ = ((byte >> 6) & 3) * 255 / 3; byte <<= 2;
-        case 1:     *out++ = ((byte >> 6) & 3) * 255 / 3;
+        default:    *out++ = ((byte >> 6) & 0x03U) * 0x55U; byte <<= 2;
+        case 3:     *out++ = ((byte >> 6) & 0x03U) * 0x55U; byte <<= 2;
+        case 2:     *out++ = ((byte >> 6) & 0x03U) * 0x55U; byte <<= 2;
+        case 1:     *out++ = ((byte >> 6) & 0x03U) * 0x55U;
+        }
+        pixels -= 4;
+    }
+}
+
+static void
+unpackL2I(UINT8* out, const UINT8* in, int pixels)
+{
+    /* nibbles (msb first, white is zero) */
+    while (pixels > 0) {
+        UINT8 byte = *in++;
+        switch (pixels) {
+        default:    *out++ = 0xFFU - (UINT8)(((byte >> 6) & 0x03U) * 0x55U); byte <<= 2;
+        case 3:     *out++ = 0xFFU - (UINT8)(((byte >> 6) & 0x03U) * 0x55U); byte <<= 2;
+        case 2:     *out++ = 0xFFU - (UINT8)(((byte >> 6) & 0x03U) * 0x55U); byte <<= 2;
+        case 1:     *out++ = 0xFFU - (UINT8)(((byte >> 6) & 0x03U) * 0x55U);
+        }
+        pixels -= 4;
+    }
+}
+
+static void
+unpackL2R(UINT8* out, const UINT8* in, int pixels)
+{
+    /* nibbles (bit order reversed, white is non-zero) */
+    while (pixels > 0) {
+        UINT8 byte = *in++;
+        byte = BITFLIP[byte];
+        switch (pixels) {
+        default:    *out++ = ((byte >> 6) & 0x03U) * 0x55U; byte <<= 2;
+        case 3:     *out++ = ((byte >> 6) & 0x03U) * 0x55U; byte <<= 2;
+        case 2:     *out++ = ((byte >> 6) & 0x03U) * 0x55U; byte <<= 2;
+        case 1:     *out++ = ((byte >> 6) & 0x03U) * 0x55U;
+        }
+        pixels -= 4;
+    }
+}
+
+static void
+unpackL2IR(UINT8* out, const UINT8* in, int pixels)
+{
+    /* nibbles (bit order reversed, white is zero) */
+    while (pixels > 0) {
+        UINT8 byte = *in++;
+        byte = BITFLIP[byte];
+        switch (pixels) {
+        default:    *out++ = 0xFFU - (UINT8)(((byte >> 6) & 0x03U) * 0x55U); byte <<= 2;
+        case 3:     *out++ = 0xFFU - (UINT8)(((byte >> 6) & 0x03U) * 0x55U); byte <<= 2;
+        case 2:     *out++ = 0xFFU - (UINT8)(((byte >> 6) & 0x03U) * 0x55U); byte <<= 2;
+        case 1:     *out++ = 0xFFU - (UINT8)(((byte >> 6) & 0x03U) * 0x55U);
         }
         pixels -= 4;
     }
@@ -207,12 +257,56 @@ unpackL2(UINT8* out, const UINT8* in, int pixels)
 static void
 unpackL4(UINT8* out, const UINT8* in, int pixels)
 {
-    /* nibbles */
+    /* nibbles (msb first, white is non-zero) */
     while (pixels > 0) {
         UINT8 byte = *in++;
         switch (pixels) {
-        default:    *out++ = ((byte >> 4) & 15) * 255 / 15; byte <<= 4;
-        case 1:     *out++ = ((byte >> 4) & 15) * 255 / 15;
+        default:    *out++ = ((byte >> 4) & 0x0FU) * 0x11U; byte <<= 4;
+        case 1:     *out++ = ((byte >> 4) & 0x0FU) * 0x11U;
+        }
+        pixels -= 2;
+    }
+}
+
+static void
+unpackL4I(UINT8* out, const UINT8* in, int pixels)
+{
+    /* nibbles (msb first, white is zero) */
+    while (pixels > 0) {
+        UINT8 byte = *in++;
+        switch (pixels) {
+        default:    *out++ = 0xFFU - (UINT8)(((byte >> 4) & 0x0FU) * 0x11U); byte <<= 4;
+        case 1:     *out++ = 0xFFU - (UINT8)(((byte >> 4) & 0x0FU) * 0x11U);
+        }
+        pixels -= 2;
+    }
+}
+
+static void
+unpackL4R(UINT8* out, const UINT8* in, int pixels)
+{
+    /* nibbles (bit order reversed, white is non-zero) */
+    while (pixels > 0) {
+        UINT8 byte = *in++;
+        byte = BITFLIP[byte];
+        switch (pixels) {
+        default:    *out++ = ((byte >> 4) & 0x0FU) * 0x11U; byte <<= 4;
+        case 1:     *out++ = ((byte >> 4) & 0x0FU) * 0x11U;
+        }
+        pixels -= 2;
+    }
+}
+
+static void
+unpackL4IR(UINT8* out, const UINT8* in, int pixels)
+{
+    /* nibbles (bit order reversed, white is zero) */
+    while (pixels > 0) {
+        UINT8 byte = *in++;
+        byte = BITFLIP[byte];
+        switch (pixels) {
+        default:    *out++ = 0xFFU - (UINT8)(((byte >> 4) & 0x0FU) * 0x11U); byte <<= 4;
+        case 1:     *out++ = 0xFFU - (UINT8)(((byte >> 4) & 0x0FU) * 0x11U);
         }
         pixels -= 2;
     }
@@ -653,6 +747,30 @@ unpackRGBa(UINT8* out, const UINT8* in, int pixels)
     }
 }
 
+static void
+unpackBGRa(UINT8* out, const UINT8* in, int pixels)
+{
+    int i;
+    /* premultiplied BGRA */
+    for (i = 0; i < pixels; i++) {
+        int a = in[3];
+        if (!a)
+            out[R] = out[G] = out[B] = out[A] = 0;
+        else if (a == 255) {
+            out[R] = in[2];
+            out[G] = in[1];
+            out[B] = in[0];
+            out[A] = a;
+        } else {
+            out[R] = CLIP(in[2] * 255 / a);
+            out[G] = CLIP(in[1] * 255 / a);
+            out[B] = CLIP(in[0] * 255 / a);
+            out[A] = a;
+        }
+        out += 4; in += 4;
+    }
+}
+
 static void
 unpackRGBAI(UINT8* out, const UINT8* in, int pixels)
 {
@@ -788,7 +906,7 @@ unpackI16N_I16B(UINT8* out, const UINT8* in, int pixels){
         C16B;
 		in += 2; tmp += 2;
     }
-	
+
 }
 static void
 unpackI16N_I16(UINT8* out, const UINT8* in, int pixels){
@@ -803,7 +921,7 @@ unpackI16N_I16(UINT8* out, const UINT8* in, int pixels){
 static void
 unpackI12_I16(UINT8* out, const UINT8* in, int pixels){
     /*  Fillorder 1/MSB -> LittleEndian, for 12bit integer greyscale tiffs.
-        
+
         According to the TIFF spec:
 
         FillOrder = 2 should be used only when BitsPerSample = 1 and
@@ -840,7 +958,7 @@ unpackI12_I16(UINT8* out, const UINT8* in, int pixels){
 #else
         out16[1] = pixel;
 #endif
-  
+
 		in += 3; out16 += 2; out+=4;
     }
     if (i == pixels-1) {
@@ -1053,7 +1171,15 @@ static struct {
 
     /* greyscale */
     {"L",       "L;2",          2,      unpackL2},
+    {"L",       "L;2I",         2,      unpackL2I},
+    {"L",       "L;2R",         2,      unpackL2R},
+    {"L",       "L;2IR",        2,      unpackL2IR},
+
     {"L",       "L;4",          4,      unpackL4},
+    {"L",       "L;4I",         4,      unpackL4I},
+    {"L",       "L;4R",         4,      unpackL4R},
+    {"L",       "L;4IR",        4,      unpackL4IR},
+
     {"L",       "L",            8,      copy1},
     {"L",       "L;I",          8,      unpackLI},
     {"L",       "L;R",          8,      unpackLR},
@@ -1104,6 +1230,7 @@ static struct {
     {"RGBA",    "LA;16B",       32,     unpackRGBALA16B},
     {"RGBA",    "RGBA",         32,     copy4},
     {"RGBA",    "RGBa",         32,     unpackRGBa},
+    {"RGBA",    "BGRa",         32,     unpackBGRa},
     {"RGBA",    "RGBA;I",       32,     unpackRGBAI},
     {"RGBA",    "RGBA;L",       32,     unpackRGBAL},
     {"RGBA",    "RGBA;15",      16,     ImagingUnpackRGBA15},
